;; Copyright 2015, 2016 Kevin Ryde

;; t-commentary.el is shared by several distributions.
;;
;; t-commentary.el is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; t-commentary.el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with this file.  If not, see <http://www.gnu.org/licenses/>.


(require 'finder)
(require 'lisp-mnt)
(eval-when-compile
  (unless (fboundp 'dolist)
    (require 'cl))) ;; for `dolist' in emacs20

(defun my-check-docstring (filename str)
  (with-temp-buffer
    (insert str)
    (let ((case-fold-search t)
          bad)

      (goto-char (point-min))
      (when (and (search-forward "internal part" nil t)
                 (re-search-forward "[-a-z0-9]*\\.el" nil (line-end-position)))
        (let ((got-filename (match-string 0)))
          (unless (equal filename got-filename)
            (message "%s:1: docstring \"internal part\" wrongly says %s\n%s"
                     filename got-filename str)
            (setq bad t))))

      (goto-char (point-min))
      (when (and (search-forward "internal part" nil t)
                 (search-forward "internal part" nil t))
        (message "%s:1: two \"internal part\"\n%s"
                 filename str)
        (setq bad t))

      bad)))

(defun my-run-test ()
  (let* ((count 0)
         bad
         (srcdir (expand-file-name ".."))
         (file-list (directory-files srcdir nil "\\`[^.].*\\.el\\'")))

    (message "srcdir is %S" srcdir)
    (when (null file-list)
      (error "Oops, no *.el files"))


    ;; temporary exclude
    (setq file-list (delete "ffap-perl-module.el" file-list))



    (dolist (basename file-list)
      (let ((fullname (expand-file-name basename srcdir)))
        (message "check %S" basename)

        (find-file fullname)

        ;;-------------------------------------------------------------------
        ;; my-check-docstring

        (goto-char (point-min))
        (let (pending)
          (while (ignore-errors (setq pending (list (read (current-buffer)))))
            (while pending
              (let ((elem (car pending)))
                (setq pending (cdr pending))
                (cond ((consp elem)
                       (setq pending (cons (car elem)
                                           (cons (cdr elem)
                                                 pending))))
                      ((stringp elem)
                       (when (my-check-docstring basename elem)
                         (setq bad t)))))))
          (setq count (1+ count)))

        ;;---------------------------------------------------------------------

        ;; the find-file
        (kill-buffer nil)))

    (when bad
      (error "docstring errors"))

    (message "  %d tests" count)
    count))
